@@ -1,28 +1,26 @@
 /*
-  $Id: drives.cpp,v 1.5 2006/03/28 03:26:16 rocky Exp $
-
-  Copyright (C) 2003, 2004, 2006 Rocky Bernstein <rocky@panix.com>
+  Copyright (C) 2003, 2004, 2006, 2008, 2009 Rocky Bernstein <rocky@gnu.org>
   
-  This program is free software; you can redistribute it and/or modify
+  This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
-  
+
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
-  
+
   You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /* Simple program to show drivers installed and what the default 
    CD-ROM drive is and what CD drives are available. */
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
+#include <cdio++/cdio.hpp>
+#include <cdio/cd_types.h>
+#include <cdio/logging.h>
+
 #ifdef HAVE_STDIO_H
 #include <stdio.h>
 #endif
@@ -32,9 +30,6 @@
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
-#include <cdio++/cdio.hpp>
-#include <cdio/cd_types.h>
-#include <cdio/logging.h>
 
 static void 
 log_handler (cdio_log_level_t level, const char message[])
