@@ -1,7 +1,6 @@
 /*
-  $Id: cdchange.c,v 1.9 2008/06/25 08:01:53 rocky Exp $
-
-  Copyright (C) 2005, 2006, 2007, 2008 Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+  Rocky Bernstein <rocky@gnu.org>
   
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -17,10 +16,16 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-/* Test media changed */
+/* config.h has to come first else _FILE_OFFSET_BITS are redefined in
+   say opensolaris. */
 #ifdef HAVE_CONFIG_H
 # include "config.h"
+# define __CDIO_CONFIG_H__ 1
 #endif
+
+#include <cdio/cdio.h>
+
+/* Test media changed */
 #include <stdio.h>
 
 #ifdef HAVE_LIMITS_H
@@ -35,6 +40,10 @@
 # include <string.h>
 #endif
 
+#ifdef HAVE_UNISTD_H
+# include <unistd.h>
+#endif
+
 #ifdef HAVE_ERRNO_H
 # include <errno.h>
 #endif 
@@ -43,31 +52,21 @@
 # include <windows.h>
 #endif
 
-#ifndef HAVE_SLEEP
-static void
-sleep(unsigned int ms)
-{
-#if defined(_WIN32)
-    Sleep(ms);
-#else
-#error sleep() unimplemented
-#endif
-}
+#ifndef HAVE_USLEEP
+#error usleep() unimplemented
 #endif
 
-#include <cdio/cdio.h>
 int
 main(int argc, const char *argv[])
 {
   CdIo_t *p_cdio;
-  unsigned long i_sleep=30;
-
+  unsigned long i_sleep_ms = (30 * 1000000);
   if (argc > 1) {
     p_cdio = cdio_open (argv[1], DRIVER_DEVICE);
     if (argc > 2) {
       errno = 0;
-      i_sleep = strtol(argv[2], (char **)NULL, 10);
-      if ( (LONG_MIN == i_sleep || LONG_MAX == i_sleep) && errno != 0 ) {
+      i_sleep_ms = strtol(argv[2], (char **)NULL, 10) * 1000000;
+      if ( (LONG_MIN == i_sleep_ms || LONG_MAX == i_sleep_ms) && errno != 0 ) {
 	printf("Invalid sleep parameter %s\n", argv[2]);
 	printf("Error reported back from strtol: %s\n", strerror(errno));
 	return 2;
@@ -87,9 +86,13 @@ main(int argc, const char *argv[])
   else 
     printf("Initial media status: not changed\n");
 
-  printf("Giving you %lu seconds to change CD if you want to do so.\n",
-	 i_sleep);
-  sleep(30);
+  printf("Giving you %g seconds to change CD if you want to do so.\n",
+	 i_sleep_ms / 1000000.0);
+  {
+      int i_ret = usleep(i_sleep_ms);
+      if (0 != i_ret) 
+	  fprintf(stderr, "Something went wrong with usleep\n");
+  }
   if (cdio_get_media_changed(p_cdio))
     printf("Media status: changed\n");
   else 
