@@ -1,7 +1,6 @@
 /*
-  $Id: isolist.c,v 1.3 2008/03/24 15:30:56 karl Exp $
-
-  Copyright (C) 2004, 2005, 2006, 2007, 2008 Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2004, 2005, 2006, 2007, 2008, 2011
+   Rocky Bernstein <rocky@gnu.org>
   
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -32,6 +31,7 @@
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
+# define __CDIO_CONFIG_H__ 1
 #endif
 #include <sys/types.h>
 #include <cdio/cdio.h>
