@@ -1,7 +1,5 @@
 /*
-  $Id: mmc3.c,v 1.2 2008/03/24 15:30:56 karl Exp $
-
-  Copyright (C) 2006, 2008 Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2006, 2008, 2009, 2011 Rocky Bernstein <rocky@gnu.org>
   
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,6 +20,7 @@
 */
 #ifdef HAVE_CONFIG_H
 # include "config.h"
+# define __CDIO_CONFIG_H__ 1
 #endif
 #include <stdio.h>
 #include <sys/types.h>
@@ -43,6 +42,7 @@ main(int argc, const char *argv[])
   driver_id_t driver_id = DRIVER_DEVICE;
   char *psz_drive = NULL;
   bool do_eject = false;
+  bool do_close = false;
   
   if (argc > 1) 
     psz_drive = strdup(argv[1]);
@@ -51,21 +51,22 @@ main(int argc, const char *argv[])
     psz_drive = cdio_get_default_device_driver(&driver_id);
     if (!psz_drive) {
       printf("Can't find a CD-ROM\n");
-     exit(1);
+     exit(77);
     }
   }
 
   p_cdio = cdio_open (psz_drive, driver_id);
   if (!p_cdio) {
     printf("Can't open %s\n", psz_drive);
-    exit(2);
+    exit(77);
   }
   
   ret = mmc_get_tray_status(p_cdio);
-  switch(ret) {
+  switch((int) ret) {
   case 0:
     printf("CD-ROM drive %s is closed.\n", psz_drive);
     do_eject = true;
+    do_close = true;
     break;
   case 1:
     printf("CD-ROM drive %s is open.\n", psz_drive);
@@ -73,11 +74,11 @@ main(int argc, const char *argv[])
   default:
     printf("Error status for drive %s: %s.\n", psz_drive,
 	   cdio_driver_errmsg(ret));
-    return 1;
+    return 77;
   }
   
   ret = mmc_get_media_changed(p_cdio);
-  switch(ret) {
+  switch((int) ret) {
   case 0:
     printf("CD-ROM drive %s media not changed since last test.\n", psz_drive);
     break;
@@ -87,16 +88,16 @@ main(int argc, const char *argv[])
   default:
     printf("Error status for drive %s: %s.\n", psz_drive, 
 	   cdio_driver_errmsg(ret));
-    return 1;
+    return 77;
   }
 
-  if (do_eject)
+  if (do_eject && argc > 2)
     ret = cdio_eject_media_drive(psz_drive);
   else
     ret = cdio_close_tray(psz_drive, &driver_id);
 
   ret = mmc_get_tray_status(p_cdio);
-  switch(ret) {
+  switch((int) ret) {
   case 0:
     printf("CD-ROM drive %s is closed.\n", psz_drive);
     break;
@@ -106,11 +107,11 @@ main(int argc, const char *argv[])
   default:
     printf("Error status for drive %s: %s.\n", psz_drive,
 	   cdio_driver_errmsg(ret));
-    return 1;
+    return 77;
   }
   
   ret = mmc_get_media_changed(p_cdio);
-  switch(ret) {
+  switch((int) ret) {
   case 0:
     printf("CD-ROM drive %s media not changed since last test.\n", psz_drive);
     break;
@@ -120,9 +121,12 @@ main(int argc, const char *argv[])
   default:
     printf("Error status for drive %s: %s.\n", psz_drive,
 	   cdio_driver_errmsg(ret));
-    return 1;
+    return 77;
   }
 
+  if (do_close)
+    cdio_close_tray(psz_drive, &driver_id);
+
   free(psz_drive);
   cdio_destroy(p_cdio);
 
