@@ -1,30 +1,24 @@
 /*
-  $Id: sample4.c,v 1.7 2005/01/04 04:40:22 rocky Exp $
-
-  Copyright (C) 2003, 2004 Rocky Bernstein <rocky@panix.com>
+  Copyright (C) 2003, 2004, 2008, 2009 Rocky Bernstein <rocky@gnu.org>
   
-  This program is free software; you can redistribute it and/or modify
+  This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
-  
+
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
-  
+
   You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /* 
    A slightly improved sample3 program: we handle cdio logging and
    take an optional CD-location.
 */
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
@@ -46,8 +40,7 @@ log_handler (cdio_log_level_t level, const char message[])
 
 static void
 print_analysis(cdio_iso_analysis_t cdio_iso_analysis, 
-	       cdio_fs_anal_t fs, int first_data, unsigned int num_audio, 
-	       track_t num_tracks, track_t first_track_num, CdIo_t *p_cdio)
+	       cdio_fs_anal_t fs, int first_data, unsigned int num_audio)
 {
   switch(CDIO_FSTYPE(fs)) {
   case CDIO_FS_AUDIO:
@@ -151,7 +144,7 @@ main(int argc, const char *argv[])
 
   if (NULL == p_cdio) {
     printf("Problem in trying to find a driver.\n\n");
-    return 1;
+    return 77;
   }
 
   first_track_num = cdio_get_first_track_num(p_cdio);
@@ -207,8 +200,7 @@ main(int argc, const char *argv[])
       
       fs = cdio_guess_cd_type(p_cdio, start_track, i, &cdio_iso_analysis);
       
-      print_analysis(cdio_iso_analysis, fs, first_data, num_audio,
-		     num_tracks, first_track_num, p_cdio);
+      print_analysis(cdio_iso_analysis, fs, first_data, num_audio);
       
       if ( !(CDIO_FSTYPE(fs) == CDIO_FS_ISO_9660 ||
 	     CDIO_FSTYPE(fs) == CDIO_FS_ISO_HFS  ||
