@@ -1,21 +1,20 @@
 /*
-  $Id: udf1.c,v 1.18 2008/01/09 04:27:16 rocky Exp $
+  $Id: udf1.c,v 1.19 2008/03/24 15:30:56 karl Exp $
 
   Copyright (C) 2005, 2008 Rocky Bernstein <rocky@gnu.org>
   
-  This program is free software; you can redistribute it and/or modify
+  This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
-  
+
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
-  
+
   You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /* Simple program to show using libudf to list files in a directory of
@@ -26,9 +25,6 @@
 #define UDF_IMAGE_PATH "../"
 #define UDF_IMAGE "/src2/cd-images/udf/UDF102ISO.iso"
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
 #include <sys/types.h>
 #include <cdio/cdio.h>
 #include <cdio/udf.h>
