@@ -1,7 +1,6 @@
 /*
-  $Id: udffile.c,v 1.3 2008/03/24 15:30:56 karl Exp $
-
-  Copyright (C) 2005, 2006, 2008 Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2005, 2006, 2008, 2009, 2010, 2011
+   Rocky Bernstein <rocky@gnu.org>
   
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,15 +23,19 @@
    things on the C++ side.
  */
 
+/* config.h has to come first else _FILE_OFFSET_BITS are redefined in
+   say opensolaris. */
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+# define __CDIO_CONFIG_H__ 1
+#endif
+
 /* This is the UDF image. */
 #define UDF_IMAGE_PATH "../"
 #define UDF_IMAGE "../test/udf102.iso"
 #define UDF_FILENAME "/COPYING"
 #define LOCAL_FILENAME "copying"
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
 #include <sys/types.h>
 #include <cdio/cdio.h>
 #include <cdio/udf.h>
