@@ -1,13 +1,13 @@
 /*
-    $Id: cdtext.h,v 1.11 2005/04/25 23:06:21 rocky Exp $
+    $Id: cdtext.h,v 1.14 2008/03/25 15:59:08 karl Exp $
 
-    Copyright (C) 2004, 2005 Rocky Bernstein <rocky@panix.com>
+    Copyright (C) 2004, 2005, 2008 Rocky Bernstein <rocky@gnu.org>
     adapted from cuetools
     Copyright (C) 2003 Svend Sanjay Sorensen <ssorensen@fastmail.fm>
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -16,8 +16,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 /*!
  * \file cdtext.h 
@@ -37,6 +36,7 @@ extern "C" {
 #endif /* __cplusplus */
 
 #define MAX_CDTEXT_FIELDS 13
+#define MIN_CDTEXT_FIELD  0
   
   /*! \brief structure for holding CD-Text information
 
@@ -46,7 +46,9 @@ extern "C" {
     char *field[MAX_CDTEXT_FIELDS];
   };
   
-  /*! \brief A list of all of the CD-Text fields */
+  /*! \brief A list of all of the CD-Text fields. Because
+    the interval has no gaps, we can use ++ to iterate over fields.
+   */
   typedef enum {
     CDTEXT_ARRANGER   =  0,   /**< name(s) of the arranger(s) */
     CDTEXT_COMPOSER   =  1,   /**< name(s) of the composer(s) */
@@ -81,8 +83,8 @@ extern "C" {
 
     The user needs to free the string when done with it.
 
-    @see cdio_get_cdtext to retrieve the cdtext structure used as
-    input here.
+    @see cdio_get_const to retrieve a constant string that doesn't
+    have to be freed.
   */
   char *cdtext_get (cdtext_field_t key, const cdtext_t *cdtext);
 
@@ -92,8 +94,8 @@ extern "C" {
     Don't use the string when the cdtext object (i.e. the CdIo_t object
     you got it from) is no longer valid.
 
-    @see cdio_get_cdtext to retrieve the cdtext structure used as
-    input here.
+    @see cdio_get to retrieve an allocated string that persists past
+    the cdtext object.
   */
   const char *cdtext_get_const (cdtext_field_t key, const cdtext_t *cdtext);
   
