@@ -1,25 +1,24 @@
 /*
-  $Id: cddap_interface.c,v 1.6 2007/10/16 15:32:02 rocky Exp $
+  $Id: cddap_interface.c,v 1.8 2008/06/13 19:26:22 flameeyes Exp $
 
-  Copyright (C) 2004, 2005, 2007 Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2004, 2005, 2007, 2008 Rocky Bernstein <rocky@gnu.org>
   Original interface.c Copyright (C) 1994-1997 
              Eissfeldt heiko@colossus.escape.de
   Current blenderization Copyright (C) 1998-1999 Monty xiphmont@mit.edu
   Copyright (C) 1998 Monty xiphmont@mit.edu
   
-  This program is free software; you can redistribute it and/or modify
+  This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
-  
+
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
-  
+
   You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 /**
  
@@ -247,12 +246,6 @@ cddap_read (cdrom_drive_t *d, void *p, lsn_t begin, long i_sectors)
   
 }
 
-/* hook */
-static int Dummy (cdrom_drive_t *d,int Switch)
-{
-  return(0);
-}
-
 static int 
 verify_read_command(cdrom_drive_t *d)
 {
@@ -288,6 +281,7 @@ verify_read_command(cdrom_drive_t *d)
 
   if(!audioflag){
     cdmessage(d,"\tCould not find any audio tracks on this disk.\n");
+    free(buff);
     return(-403);
   }
 
@@ -388,7 +382,7 @@ cddap_init_drive (cdrom_drive_t *d)
   } 	 
 #endif /*HAVE_LINUX_MAJOR_H*/
 
-  d->enable_cdda = Dummy;
+  d->enable_cdda = dummy_exception;
   d->set_speed   = cddap_setspeed;
   d->read_toc    = cddap_readtoc;
   d->read_audio  = cddap_read;
