@@ -1,7 +1,5 @@
 /*
-  $Id: cdio_assert.h,v 1.2 2008/04/22 15:29:11 karl Exp $
-
-  Copyright (C) 2008 Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2008, 2011 Rocky Bernstein <rocky@gnu.org>
   Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
 
   This program is free software: you can redistribute it and/or modify
@@ -23,8 +21,9 @@
 
 #if defined(__GNUC__)
 
-#ifdef HAVE_CONFIG_H
+#if defined(HAVE_CONFIG_H) && !defined(__CDIO_CONFIG_H__)
 # include "config.h"
+# define __CDIO_CONFIG_H__ 1
 #endif
 
 #include <cdio/types.h>
