@@ -1,7 +1,6 @@
 /*
-  $Id: device.c,v 1.41 2008/04/22 15:29:12 karl Exp $
-
-  Copyright (C) 2005, 2006, 2008 Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2005, 2006, 2008, 2011
+  Rocky Bernstein <rocky@gnu.org>
 
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -21,12 +20,14 @@
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
+# define __CDIO_CONFIG_H__ 1
 #endif
 
 #include <cdio/cdio.h>
 #include <cdio/cd_types.h>
 #include <cdio/logging.h>
 #include "cdio_private.h"
+#include <cdio/mmc_cmds.h>
 
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
@@ -429,6 +430,7 @@ cdio_close_tray (const char *psz_orig_drive, /*in/out*/ driver_id_t
         return drc;
       }
     }
+    free(psz_drive);
     return DRIVER_OP_UNSUPPORTED;
   }
   
@@ -439,6 +441,8 @@ cdio_close_tray (const char *psz_orig_drive, /*in/out*/ driver_id_t
     free(psz_drive);
     return drc;
   }
+  
+  free(psz_drive);
   return DRIVER_OP_UNSUPPORTED;
 }
 
@@ -1044,7 +1048,7 @@ driver_return_code_t
 cdio_set_blocksize ( const CdIo_t *p_cdio, int i_blocksize )
 {
   if (!p_cdio) return DRIVER_OP_UNINIT;
-  if (p_cdio->op.set_blocksize) return DRIVER_OP_UNSUPPORTED;
+  if (!p_cdio->op.set_blocksize) return DRIVER_OP_UNSUPPORTED;
   return p_cdio->op.set_blocksize(p_cdio->env, i_blocksize);
 }
 
