@@ -1,30 +1,27 @@
 /*
-    $Id: osx.c,v 1.11 2007/08/11 12:28:25 flameeyes Exp $
-
-    Copyright (C) 2003, 2004, 2005, 2006 Rocky Bernstein 
-    <rockyb@users.sourceforge.net> 
-    from vcdimager code: 
-    Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
-    and VideoLAN code Copyright (C) 1998-2001 VideoLAN
+  Copyright (C) 2003, 2004, 2005, 2006, 2008, 2010, 2011
+   Rocky Bernstein <rocky@gnu.org> 
+  from vcdimager code: 
+  Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
+  and VideoLAN code Copyright (C) 1998-2001 VideoLAN
       Authors: Johan Bilien <jobi@via.ecp.fr>
                Gildas Bazin <gbazin@netcourrier.com>
                Jon Lech Johansen <jon-vl@nanocrew.net>
                Derk-Jan Hartman <hartman at videolan.org>
                Justin F. Hallett <thesin@southofheaven.org>
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+  This program is free software: you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or
+  (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /* This file contains OSX-specific code and implements low-level 
@@ -32,10 +29,11 @@
 */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+# include "config.h"
+# define __CDIO_CONFIG_H__ 1
 #endif
 
-static const char _rcsid[] = "$Id: osx.c,v 1.11 2007/08/11 12:28:25 flameeyes Exp $";
+static const char _rcsid[] = "$Id: osx.c,v 1.14 2008/10/17 11:58:52 rocky Exp $";
 
 #include <cdio/logging.h>
 #include <cdio/sector.h>
@@ -147,10 +145,10 @@ static bool read_toc_osx (void *p_user_data);
 static track_format_t get_track_format_osx(void *p_user_data, 
                                            track_t i_track);
 
-/****
+/**
  * GetRegistryEntryProperties - Gets the registry entry properties for
  *  an io_service_t.
- *****/
+ */
 
 static CFMutableDictionaryRef
 GetRegistryEntryProperties ( io_service_t service )
@@ -248,43 +246,38 @@ get_scsi(_img_private_t *p_env)
 
 static bool 
 init_osx(_img_private_t *p_env) {
-  mach_port_t port;
   char *psz_devname;
   kern_return_t ret;
   io_iterator_t iterator;
-  
-  p_env->gen.fd = open( p_env->gen.source_name, O_RDONLY | O_NONBLOCK );
+
+  /* Only open if not already opened. Otherwise, too many descriptors
+     are holding the device busy.  */
+  if (-1 == p_env->gen.fd)
+    p_env->gen.fd = open( p_env->gen.source_name, O_RDONLY | O_NONBLOCK );
+
   if (-1 == p_env->gen.fd) {
     cdio_warn("Failed to open %s: %s", p_env->gen.source_name,
                strerror(errno));
     return false;
   }
 
-  /* get the device name */
+  /* Get the device name. */
   psz_devname = strrchr( p_env->gen.source_name, '/');
   if( NULL != psz_devname )
     ++psz_devname;
   else
     psz_devname = p_env->gen.source_name;
   
-  /* unraw the device name */
+  /* Unraw the device name. */
   if( *psz_devname == 'r' )
     ++psz_devname;
   
-  /* get port for IOKit communication */
-  ret = IOMasterPort( MACH_PORT_NULL, &port );
-  
-  if( ret != KERN_SUCCESS )
-    {
-      cdio_warn( "IOMasterPort: 0x%08x", ret );
-      return false;
-    }
-  
-  ret = IOServiceGetMatchingServices( port, 
-                                      IOBSDNameMatching(port, 0, psz_devname),
+  ret = IOServiceGetMatchingServices( kIOMasterPortDefault, 
+                                      IOBSDNameMatching(kIOMasterPortDefault, 
+							0, psz_devname),
                                       &iterator );
   
-  /* get service iterator for the device */
+  /* Get service iterator for the device. */
   if( ret != KERN_SUCCESS )
     {
         cdio_warn( "IOServiceGetMatchingServices: 0x%08x", ret );
@@ -294,7 +287,7 @@ init_osx(_img_private_t *p_env) {
   /* first service */
   p_env->MediaClass_service = IOIteratorNext( iterator );
   IOObjectRelease( iterator );
-  
+
   /* search for kIOCDMediaClass or kIOCDVDMediaClass */ 
   while( p_env->MediaClass_service && 
          (!IOObjectConformsTo(p_env->MediaClass_service, kIOCDMediaClass)) &&
@@ -334,7 +327,7 @@ init_osx(_img_private_t *p_env) {
 #endif
 }
 
-/*!
+/**
   Run a SCSI MMC command. 
  
   cdio          CD structure set by cdio_open().
@@ -352,7 +345,6 @@ init_osx(_img_private_t *p_env) {
 #if 1
 
 /* process a complete scsi command. */
-// handle_scsi_cmd(cdrom_drive *d,
 static int 
 run_mmc_cmd_osx( void *p_user_data, 
                  unsigned int i_timeout_ms,
@@ -368,11 +360,15 @@ run_mmc_cmd_osx( void *p_user_data,
 
   if (!p_env->scsi_task) return DRIVER_OP_UNSUPPORTED;
 
+  p_env->gen.scsi_mmc_sense_valid = 0;
   memcpy(cmdbuf, p_cdb, i_cdb);
 
-  dir = ( SCSI_MMC_DATA_READ == e_direction)
-    ? kSCSIDataTransfer_FromTargetToInitiator
-    : kSCSIDataTransfer_FromInitiatorToTarget;
+  dir = 
+      (SCSI_MMC_DATA_READ == e_direction) 
+      ? kSCSIDataTransfer_FromTargetToInitiator :
+      (SCSI_MMC_DATA_WRITE == e_direction) 
+      ? kSCSIDataTransfer_FromInitiatorToTarget
+      : kSCSIDataTransfer_NoDataTransfer;
   
   if (!i_buf)
     dir = kSCSIDataTransfer_NoDataTransfer;
@@ -420,6 +416,8 @@ run_mmc_cmd_osx( void *p_user_data,
       fprintf(stderr, "%02x ", cmdbuf[i]);
     
     fprintf(stderr, "\n");
+    memcpy((void *) p_env->gen.scsi_mmc_sense, &p_env->sense, kSenseDefaultSize);
+
     return TR_UNKNOWN;
   }
   
@@ -471,7 +469,7 @@ run_mmc_cmd_osx( void *p_user_data,
 #endif
 
 #if 0
-/*!
+/**
   Run a SCSI MMC command. 
  
   cdio          CD structure set by cdio_open().
@@ -629,7 +627,7 @@ GetFeaturesFlagsForDrive ( CFDictionaryRef dict,
   return true;
 }
 
-/*! 
+/**
   Get disc type associated with the cd object.
 */
 static discmode_t
@@ -876,10 +874,13 @@ static void
 _free_osx (void *p_user_data) {
   _img_private_t *p_env = p_user_data;
   if (NULL == p_env) return;
+  if (p_env->gen.fd != -1)
+    close(p_env->gen.fd);
+  if (p_env->MediaClass_service)
+    IOObjectRelease( p_env->MediaClass_service );
   cdio_generic_free(p_env);
   if (NULL != p_env->pp_lba)  free((void *) p_env->pp_lba);
   if (NULL != p_env->pTOC)    free((void *) p_env->pTOC);
-  IOObjectRelease( p_env->MediaClass_service );
 
   if (p_env->scsi_task)
     (*p_env->scsi_task)->Release(p_env->scsi_task);
@@ -897,11 +898,9 @@ _free_osx (void *p_user_data) {
   if (p_env->plugin) 
     IODestroyPlugInInterface(p_env->plugin);
 
-  IOObjectRelease(p_env->MediaClass_service);
-
 }
 
-/*!
+/**
    Reads i_blocks of data sectors from cd device into p_data starting
    from i_lsn.
    Returns DRIVER_OP_SUCCESS if no error. 
@@ -950,7 +949,7 @@ read_data_sectors_osx (void *p_user_data, void *p_data, lsn_t i_lsn,
 }
 
   
-/*!
+/**
    Reads i_blocks of mode2 form2 sectors from cd device into data starting
    from i_lsn.
    Returns 0 if no error. 
@@ -984,7 +983,7 @@ read_mode1_sectors_osx (void *p_user_data, void *p_data, lsn_t i_lsn,
   return DRIVER_OP_SUCCESS;
 }
 
-/*!
+/**
    Reads i_blocks of mode2 form2 sectors from cd device into data starting
    from lsn.
    Returns DRIVER_OP_SUCCESS if no error. 
@@ -1020,7 +1019,7 @@ read_mode2_sectors_osx (void *p_user_data, void *p_data, lsn_t i_lsn,
 }
 
   
-/*!
+/**
    Reads a single audio sector from CD device into p_data starting from lsn.
    Returns 0 if no error. 
  */
@@ -1049,7 +1048,7 @@ read_audio_sectors_osx (void *user_data, void *p_data, lsn_t lsn,
   return DRIVER_OP_SUCCESS;
 }
 
-/*!
+/**
    Reads a single mode2 sector from cd device into p_data starting
    from lsn. Returns 0 if no error. 
  */
@@ -1060,7 +1059,7 @@ read_mode1_sector_osx (void *p_user_data, void *p_data, lsn_t i_lsn,
   return read_mode1_sectors_osx(p_user_data, p_data, i_lsn, b_form2, 1);
 }
 
-/*!
+/**
    Reads a single mode2 sector from cd device into p_data starting
    from lsn. Returns 0 if no error. 
  */
@@ -1071,7 +1070,7 @@ read_mode2_sector_osx (void *p_user_data, void *p_data, lsn_t i_lsn,
   return read_mode2_sectors_osx(p_user_data, p_data, i_lsn, b_form2, 1);
 }
 
-/*!
+/**
   Set the key "arg" to "value" in source device.
 */
 static driver_return_code_t
@@ -1143,7 +1142,7 @@ TestDevice(_img_private_t *p_env, io_service_t service)
 }
 #endif
 
-/*! 
+/**
   Read and cache the CD's Track Table of Contents and track info.
   Return false if successful or true if an error.
 */
@@ -1175,10 +1174,12 @@ read_toc_osx (void *p_user_data)
       CFDataGetBytes( data, range, (u_char *) p_env->pTOC );
     } else {
       cdio_warn( "Trouble allocating CDROM TOC" );
+      CFRelease( propertiesDict );    
       return false;
     }
   } else     {
     cdio_warn( "Trouble reading TOC" );
+    CFRelease( propertiesDict );    
     return false;
   }
 
@@ -1293,7 +1294,7 @@ read_toc_osx (void *p_user_data)
 
 }
 
-/*!  
+/**
   Return the starting LSN track number
   i_track in obj.  Track numbers start at 1.
   The "leadout" track is specified either by
@@ -1317,7 +1318,7 @@ get_track_lba_osx(void *p_user_data, track_t i_track)
   }
 }
 
-/*!
+/**
   Eject media . Return DRIVER_OP_SUCCESS if successful.
 
   The only way to cleanly unmount the disc under MacOS X (before
@@ -1393,16 +1394,19 @@ static void media_eject_callback(DADiskRef disk, DADissenterRef dissenter, void
     if ( dissenter )
       {
 	CFStringRef status = DADissenterGetStatusString(dissenter);
-	size_t cstr_size = CFStringGetLength(status);
-	char *cstr = malloc(cstr_size);
-	if ( CFStringGetCString( status,
+	if (status)
+	{ 
+		size_t cstr_size = CFStringGetLength(status);
+		char *cstr = malloc(cstr_size);
+		if ( CFStringGetCString( status,
 				 cstr, cstr_size,
 				 kCFStringEncodingASCII ) )
-	  CFRelease( status );
+	  	CFRelease( status );
 
-	cdio_warn("%s", cstr);
+		cdio_warn("%s", cstr);
 
-	free(cstr);
+		free(cstr);
+	}
       }
 
     dacontext->result    = (dissenter ? DRIVER_OP_ERROR : DRIVER_OP_SUCCESS);
@@ -1443,6 +1447,10 @@ _eject_media_osx (void *user_data) {
       return DRIVER_OP_ERROR;
     }
 
+  if (p_env->gen.fd != -1)
+    close(p_env->gen.fd);
+  p_env->gen.fd = -1;
+
   dacontext.result    = DRIVER_OP_UNINIT;
   dacontext.completed = FALSE;
   dacontext.runloop   = CFRunLoopGetCurrent();
@@ -1494,7 +1502,7 @@ _eject_media_osx (void *user_data) {
 }
 #endif
 
-/*!
+/**
    Return the size of the CD in logical block address (LBA) units.
  */
 static lsn_t
@@ -1503,7 +1511,7 @@ get_disc_last_lsn_osx (void *user_data)
   return get_track_lba_osx(user_data, CDIO_CDROM_LEADOUT_TRACK);
 }
 
-/*!
+/**
   Return the value associated with the key "arg".
 */
 static const char *
@@ -1524,7 +1532,7 @@ _get_arg_osx (void *user_data, const char key[])
   return NULL;
 }
 
-/*!
+/**
   Return the media catalog number MCN.
  */
 static char *
@@ -1543,7 +1551,7 @@ get_mcn_osx (const void *user_data) {
 }
 
 
-/*!  
+/**
   Get format of track. 
 */
 static track_format_t
@@ -1590,7 +1598,7 @@ get_track_format_osx(void *p_user_data, track_t i_track)
 
 }
 
-/*!
+/**
   Return true if we have XA data (green, mode2 form1) or
   XA data (green, mode2 form2). That is track begins:
   sync - header - subheader
@@ -1642,7 +1650,7 @@ set_speed_osx (void *p_user_data, int i_speed)
 
 #endif /* HAVE_DARWIN_CDROM */
 
-/*!
+/**
   Close tray on CD-ROM.
   
   @param psz_drive the CD-ROM drive to be closed.
@@ -1694,7 +1702,7 @@ close_tray_osx (const char *psz_drive)
 #endif /*HAVE_DARWIN_CDROM*/
 }
 
-/*!
+/**
   Return a string containing the default CD device if none is specified.
  */
 char **
@@ -1717,7 +1725,7 @@ cdio_get_devices_osx(void)
       return( NULL );
     }
   
-  classes_to_match = IOServiceMatching( kIOCDMediaClass );
+  classes_to_match = IOServiceMatching( kIOMediaClass );
   if( classes_to_match == NULL )
     {
       return( NULL );
@@ -1778,7 +1786,7 @@ cdio_get_devices_osx(void)
 #endif /* HAVE_DARWIN_CDROM */
 }
 
-/*!
+/**
   Return a string containing the default CD device if none is specified.
  */
 char *
@@ -1788,18 +1796,11 @@ cdio_get_default_device_osx(void)
   return NULL;
 #else
   io_object_t   next_media;
-  mach_port_t   master_port;
   kern_return_t kern_result;
   io_iterator_t media_iterator;
   CFMutableDictionaryRef classes_to_match;
   
-  kern_result = IOMasterPort( MACH_PORT_NULL, &master_port );
-  if( kern_result != KERN_SUCCESS )
-    {
-      return( NULL );
-    }
-  
-  classes_to_match = IOServiceMatching( kIOCDMediaClass );
+  classes_to_match = IOServiceMatching( kIOMediaClass );
   if( classes_to_match == NULL )
     {
       return( NULL );
@@ -1808,7 +1809,7 @@ cdio_get_default_device_osx(void)
   CFDictionarySetValue( classes_to_match, CFSTR(kIOMediaEjectableKey),
                         kCFBooleanTrue );
   
-  kern_result = IOServiceGetMatchingServices( master_port, 
+  kern_result = IOServiceGetMatchingServices( kIOMasterPortDefault, 
                                               classes_to_match,
                                               &media_iterator );
   if( kern_result != KERN_SUCCESS )
@@ -1859,7 +1860,7 @@ cdio_get_default_device_osx(void)
 #endif /* HAVE_DARWIN_CDROM */
 }
 
-/*!
+/**
   Initialization routine. This is the only thing that doesn't
   get called via a function pointer. In fact *we* are the
   ones to set that up.
@@ -1875,7 +1876,7 @@ cdio_open_am_osx (const char *psz_source_name, const char *psz_access_mode)
 }
 
 
-/*!
+/**
   Initialization routine. This is the only thing that doesn't
   get called via a function pointer. In fact *we* are the
   ones to set that up.
@@ -1883,7 +1884,6 @@ cdio_open_am_osx (const char *psz_source_name, const char *psz_access_mode)
 CdIo_t *
 cdio_open_osx (const char *psz_orig_source)
 {
-
 #ifdef HAVE_DARWIN_CDROM
   CdIo_t *ret;
   _img_private_t *_data;
@@ -1935,7 +1935,10 @@ cdio_open_osx (const char *psz_orig_source)
 
   if (NULL == psz_orig_source) {
     psz_source=cdio_get_default_device_osx();
-    if (NULL == psz_source) return NULL;
+    if (NULL == psz_source) { 
+ 	cdio_generic_free(_data); 
+ 	return NULL; 
+    } 
     _set_arg_osx(_data, "source", psz_source);
     free(psz_source);
   } else {
@@ -1946,16 +1949,18 @@ cdio_open_osx (const char *psz_orig_source)
 #if 0
       cdio_info ("source %s is a not a device", psz_orig_source);
 #endif
+      cdio_generic_free(_data); 
       return NULL;
     }
   }
 
   ret = cdio_new ((void *)_data, &_funcs);
-  if (ret == NULL) return NULL;
-
+  if (ret == NULL) {
+    cdio_generic_free(_data);
+    return NULL; 
+  } 
   ret->driver_id = DRIVER_OSX;
-
-  if (cdio_generic_init(_data, O_RDONLY) && init_osx(_data))
+  if (cdio_generic_init(_data, O_RDONLY | O_NONBLOCK) && init_osx(_data))
     return ret;
   else {
     cdio_generic_free (_data);
