@@ -1,5 +1,5 @@
 #!/bin/sh
-#$Id: check_iso.sh.in,v 1.14 2007/12/28 02:11:01 rocky Exp $
+#$Id: check_iso.sh.in,v 1.15 2008/10/17 01:51:47 rocky Exp $
 
 if test -z $srcdir ; then
   srcdir=`pwd`
@@ -18,24 +18,24 @@ fi
 BASE=`basename $0 .sh`
 fname=copying
 
-opts="--quiet ${srcdir}/${fname}.iso --iso9660 "
+opts="--quiet ${srcdir}/data/${fname}.iso --iso9660 "
 test_iso_info  "$opts" ${fname}.dump ${srcdir}/${fname}.right
 RC=$?
 check_result $RC 'iso-info basic test' "$ISO_INFO $opts"
 
-opts="--ignore --image ${srcdir}/${fname}.iso --extract $fname "
+opts="--ignore --image ${srcdir}/data/${fname}.iso --extract $fname "
 test_iso_read  "$opts" ${fname} ${srcdir}/copying.gpl
 RC=$?
 check_result $RC 'iso-read basic test' "$ISO_READ $opts"
 
-if test -n "1"; then
+if test -n ""; then
   fname=copying-rr
-  opts="--quiet ${srcdir}/${fname}.iso --iso9660 "
+  opts="--quiet ${srcdir}/data/${fname}.iso --iso9660 "
   test_iso_info  "$opts" ${fname}.dump ${srcdir}/${fname}.right
   RC=$?
   check_result $RC 'iso-info Rock Ridge test' "$ISO_INFO $opts"
 
-  opts="--image ${srcdir}/${fname}.iso --extract COPYING"
+  opts="--image ${srcdir}/data/${fname}.iso --extract COPYING"
   test_iso_read  "$opts" ${fname} ${srcdir}/copying-rr.gpl
   RC=$?
   check_result $RC 'iso-read RR test' "$ISO_READ $opts"
@@ -44,11 +44,11 @@ fi
 if test -n "1" ; then
   BASE=`basename $0 .sh`
   fname=joliet
-  opts="--quiet ${srcdir}/${fname}.iso --iso9660 "
+  opts="--quiet ${srcdir}/data/${fname}.iso --iso9660 "
   test_iso_info  "$opts" ${fname}-nojoliet.dump ${srcdir}/${fname}.right
   RC=$?
   check_result $RC 'iso-info Joliet test' "$cmdline"
-  opts="--quiet ${srcdir}/${fname}.iso --iso9660 --no-joliet "
+  opts="--quiet ${srcdir}/data/${fname}.iso --iso9660 --no-joliet "
   test_iso_info  "$opts" ${fname}-nojoliet.dump \
                  ${srcdir}/${fname}-nojoliet.right
   RC=$?
