@@ -1,25 +1,25 @@
 /*
- * Copyright (C) 2005, 2006 Rocky Bernstein <rocky@panix.com>
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2, or (at your option)
- *   any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- */
+  Copyright (C) 2005, 2006, 2008, 2011 Rocky Bernstein <rocky@gnu.org>
+
+  This program is free software: you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or
+  (at your option) any later version.
+
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
+
 /* Simple program to show using libcdio's version of cdparanoia. */
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
+# define __CDIO_CONFIG_H__ 1
 #endif
 
 #include <cdio/paranoia.h>
@@ -43,9 +43,9 @@
 #define SKIP_TEST_RC 77
 
 #define MAX_SECTORS 50
-uint8_t audio_buf[MAX_SECTORS][CDIO_CD_FRAMESIZE_RAW] = { {0}, };
-paranoia_cb_mode_t audio_status[MAX_SECTORS];
-unsigned int i = 0;
+static uint8_t audio_buf[MAX_SECTORS][CDIO_CD_FRAMESIZE_RAW] = { {0}, };
+static paranoia_cb_mode_t audio_status[MAX_SECTORS];
+static unsigned int i = 0;
 
 static void 
 callback(long int inpos, paranoia_cb_mode_t function)
@@ -132,8 +132,8 @@ main(int argc, const char *argv[])
 	  if (psz_mes || psz_err)
 	    printf("%s%s\n", psz_mes ? psz_mes: "", psz_err ? psz_err: "");
 	  
-	  if (psz_err) free(psz_err);
-	  if (psz_mes) free(psz_mes);
+	  free(psz_err);
+	  free(psz_mes);
 	  if( !p_readbuf ) {
 	    printf("paranoia read error. Stopping.\n");
 	    goto out;
